# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PhysicalFormat'
        db.create_table('physical_formats', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['PhysicalFormat'])

        # Adding model 'PublishingStatus'
        db.create_table('publishing_status', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['PublishingStatus'])

        # Adding model 'AcquisitionMode'
        db.create_table('acquisition_modes', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['AcquisitionMode'])

        # Adding model 'MetadataAuthor'
        db.create_table('metadata_authors', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['MetadataAuthor'])

        # Adding model 'MetadataWriter'
        db.create_table('metadata_writers', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['MetadataWriter'])

        # Adding model 'LegalRight'
        db.create_table('legal_rights', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['LegalRight'])

        # Adding model 'RecordingContext'
        db.create_table('recording_contexts', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['RecordingContext'])

        # Adding model 'AdConversion'
        db.create_table('ad_conversions', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['AdConversion'])

        # Adding model 'VernacularStyle'
        db.create_table('vernacular_styles', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['VernacularStyle'])

        # Adding model 'GenericStyle'
        db.create_table('generic_styles', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['GenericStyle'])

        # Adding model 'ContextKeyword'
        db.create_table('context_keywords', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['ContextKeyword'])

        # Adding model 'Publisher'
        db.create_table('publishers', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['Publisher'])

        # Adding model 'PublisherCollection'
        db.create_table('publisher_collections', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('publisher', self.gf('telemeta.models.core.ForeignKey')(related_name='publisher_collections', to=orm['telemeta.Publisher'])),
            ('value', self.gf('telemeta.models.core.CharField')(max_length=250)),
        ))
        db.send_create_signal('telemeta', ['PublisherCollection'])

        # Adding model 'EthnicGroup'
        db.create_table('ethnic_groups', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
        ))
        db.send_create_signal('telemeta', ['EthnicGroup'])

        # Adding model 'EthnicGroupAlias'
        db.create_table('ethnic_group_aliases', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ethnic_group', self.gf('telemeta.models.core.ForeignKey')(related_name='aliases', to=orm['telemeta.EthnicGroup'])),
            ('value', self.gf('telemeta.models.core.CharField')(max_length=250)),
        ))
        db.send_create_signal('telemeta', ['EthnicGroupAlias'])

        # Adding unique constraint on 'EthnicGroupAlias', fields ['ethnic_group', 'value']
        db.create_unique('ethnic_group_aliases', ['ethnic_group_id', 'value'])

        # Adding model 'Location'
        db.create_table('locations', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('telemeta.models.core.CharField')(unique=True, max_length=150)),
            ('type', self.gf('telemeta.models.core.IntegerField')(default=0, db_index=True, blank=True)),
            ('complete_type', self.gf('telemeta.models.core.ForeignKey')(related_name='locations', to=orm['telemeta.LocationType'])),
            ('current_location', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='past_names', null=True, blank=True, to=orm['telemeta.Location'])),
            ('latitude', self.gf('telemeta.models.core.FloatField')(default=None, null=True, blank=True)),
            ('longitude', self.gf('telemeta.models.core.FloatField')(default=None, null=True, blank=True)),
            ('is_authoritative', self.gf('telemeta.models.core.BooleanField')(default=False)),
        ))
        db.send_create_signal('telemeta', ['Location'])

        # Adding model 'LocationType'
        db.create_table('location_types', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('telemeta.models.core.CharField')(unique=True, max_length=64)),
            ('name', self.gf('telemeta.models.core.CharField')(max_length=150)),
        ))
        db.send_create_signal('telemeta', ['LocationType'])

        # Adding model 'LocationAlias'
        db.create_table('location_aliases', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('location', self.gf('telemeta.models.core.ForeignKey')(related_name='aliases', to=orm['telemeta.Location'])),
            ('alias', self.gf('telemeta.models.core.CharField')(max_length=150)),
            ('is_authoritative', self.gf('telemeta.models.core.BooleanField')(default=False)),
        ))
        db.send_create_signal('telemeta', ['LocationAlias'])

        # Adding unique constraint on 'LocationAlias', fields ['location', 'alias']
        db.create_unique('location_aliases', ['location_id', 'alias'])

        # Adding model 'LocationRelation'
        db.create_table('location_relations', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('location', self.gf('telemeta.models.core.ForeignKey')(related_name='ancestor_relations', to=orm['telemeta.Location'])),
            ('ancestor_location', self.gf('telemeta.models.core.ForeignKey')(related_name='descendant_relations', to=orm['telemeta.Location'])),
            ('is_direct', self.gf('telemeta.models.core.BooleanField')(default=False, db_index=True)),
            ('is_authoritative', self.gf('telemeta.models.core.BooleanField')(default=False)),
        ))
        db.send_create_signal('telemeta', ['LocationRelation'])

        # Adding unique constraint on 'LocationRelation', fields ['location', 'ancestor_location']
        db.create_unique('location_relations', ['location_id', 'ancestor_location_id'])

        # Adding model 'Revision'
        db.create_table('revisions', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('element_type', self.gf('telemeta.models.core.CharField')(max_length=16)),
            ('element_id', self.gf('telemeta.models.core.IntegerField')()),
            ('change_type', self.gf('telemeta.models.core.CharField')(max_length=16)),
            ('time', self.gf('telemeta.models.core.DateTimeField')(default=None, auto_now_add=True, null=True, blank=True)),
            ('user', self.gf('telemeta.models.core.ForeignKey')(related_name='revisions', db_column='username', to=orm['auth.User'])),
        ))
        db.send_create_signal('telemeta', ['Revision'])

        # Adding model 'UserProfile'
        db.create_table('profiles', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('telemeta.models.core.ForeignKey')(to=orm['auth.User'], unique=True)),
            ('institution', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('function', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('address', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('telephone', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('expiration_date', self.gf('telemeta.models.core.DateField')(default=None, null=True, blank=True)),
        ))
        db.send_create_signal('telemeta', ['UserProfile'])

        # Adding model 'Instrument'
        db.create_table('instruments', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('telemeta.models.core.CharField')(max_length=250)),
        ))
        db.send_create_signal('telemeta', ['Instrument'])

        # Adding model 'InstrumentAlias'
        db.create_table('instrument_aliases', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('telemeta.models.core.CharField')(max_length=250)),
        ))
        db.send_create_signal('telemeta', ['InstrumentAlias'])

        # Adding model 'InstrumentRelation'
        db.create_table('instrument_relations', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('instrument', self.gf('telemeta.models.core.ForeignKey')(related_name='parent_relation', to=orm['telemeta.Instrument'])),
            ('parent_instrument', self.gf('telemeta.models.core.ForeignKey')(related_name='child_relation', to=orm['telemeta.Instrument'])),
        ))
        db.send_create_signal('telemeta', ['InstrumentRelation'])

        # Adding unique constraint on 'InstrumentRelation', fields ['instrument', 'parent_instrument']
        db.create_unique('instrument_relations', ['instrument_id', 'parent_instrument_id'])

        # Adding model 'InstrumentAliasRelation'
        db.create_table('instrument_alias_relations', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('alias', self.gf('telemeta.models.core.ForeignKey')(related_name='other_name', to=orm['telemeta.InstrumentAlias'])),
            ('instrument', self.gf('telemeta.models.core.ForeignKey')(related_name='relation', to=orm['telemeta.InstrumentAlias'])),
        ))
        db.send_create_signal('telemeta', ['InstrumentAliasRelation'])

        # Adding unique constraint on 'InstrumentAliasRelation', fields ['alias', 'instrument']
        db.create_unique('instrument_alias_relations', ['alias_id', 'instrument_id'])

        # Adding model 'Language'
        db.create_table('languages', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('identifier', self.gf('telemeta.models.core.CharField')(default='', max_length=3, blank=True)),
            ('part2B', self.gf('telemeta.models.core.CharField')(default='', max_length=3, blank=True)),
            ('part2T', self.gf('telemeta.models.core.CharField')(default='', max_length=3, blank=True)),
            ('part1', self.gf('telemeta.models.core.CharField')(default='', max_length=1, blank=True)),
            ('scope', self.gf('telemeta.models.core.CharField')(default='', max_length=1, blank=True)),
            ('type', self.gf('telemeta.models.core.CharField')(default='', max_length=1, blank=True)),
            ('name', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('comment', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
        ))
        db.send_create_signal('telemeta', ['Language'])

        # Adding model 'MediaCorpus'
        db.create_table('media_corpus', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('reference', self.gf('telemeta.models.core.CharField')(default=None, max_length=250, unique=True, null=True, blank=True)),
            ('title', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('description', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('code', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
            ('public_access', self.gf('telemeta.models.core.CharField')(default='metadata', max_length=16, blank=True)),
        ))
        db.send_create_signal('telemeta', ['MediaCorpus'])

        # Adding model 'MediaCorpusCollectionRelation'
        db.create_table('media_corpus_collection_relations', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('collection', self.gf('telemeta.models.core.ForeignKey')(related_name='parent_relation', to=orm['telemeta.MediaCollection'])),
            ('corpus', self.gf('telemeta.models.core.ForeignKey')(related_name='child_relation', to=orm['telemeta.MediaCorpus'])),
        ))
        db.send_create_signal('telemeta', ['MediaCorpusCollectionRelation'])

        # Adding unique constraint on 'MediaCorpusCollectionRelation', fields ['collection', 'corpus']
        db.create_unique('media_corpus_collection_relations', ['collection_id', 'corpus_id'])

        # Adding model 'MediaCollection'
        db.create_table('media_collections', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('reference', self.gf('telemeta.models.core.CharField')(default=None, max_length=250, unique=True, null=True, blank=True)),
            ('title', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('alt_title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('creator', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('recording_context', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.RecordingContext'])),
            ('recorded_from_year', self.gf('telemeta.models.core.IntegerField')(default=0, blank=True)),
            ('recorded_to_year', self.gf('telemeta.models.core.IntegerField')(default=0, blank=True)),
            ('year_published', self.gf('telemeta.models.core.IntegerField')(default=0, blank=True)),
            ('collector', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('publisher', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.Publisher'])),
            ('publisher_collection', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.PublisherCollection'])),
            ('publisher_serial', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('booklet_author', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('external_references', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('doctype_code', self.gf('telemeta.models.core.IntegerField')(default=0, blank=True)),
            ('public_access', self.gf('telemeta.models.core.CharField')(default='metadata', max_length=16, blank=True)),
            ('legal_rights', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.LegalRight'])),
            ('acquisition_mode', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.AcquisitionMode'])),
            ('cnrs_contributor', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('metadata_author', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.MetadataAuthor'])),
            ('booklet_description', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('publishing_status', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.PublishingStatus'])),
            ('alt_ids', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('comment', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('metadata_writer', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.MetadataWriter'])),
            ('travail', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('items_done', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('collector_is_creator', self.gf('telemeta.models.core.BooleanField')(default=False)),
            ('is_published', self.gf('telemeta.models.core.BooleanField')(default=False)),
            ('conservation_site', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('code', self.gf('telemeta.models.core.CharField')(unique=True, max_length=250)),
            ('old_code', self.gf('telemeta.models.core.CharField')(default=None, max_length=250, null=True, blank=True)),
            ('approx_duration', self.gf('telemeta.models.core.DurationField')(default='0', blank=True)),
            ('physical_items_num', self.gf('telemeta.models.core.IntegerField')(default=0, blank=True)),
            ('physical_format', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.PhysicalFormat'])),
            ('ad_conversion', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='collections', null=True, blank=True, to=orm['telemeta.AdConversion'])),
            ('state', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('a_informer_07_03', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
        ))
        db.send_create_signal('telemeta', ['MediaCollection'])

        # Adding model 'MediaCollectionRelated'
        db.create_table('media_collection_related', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('date', self.gf('telemeta.models.core.DateTimeField')(default=None, auto_now=True, null=True, blank=True)),
            ('description', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('mime_type', self.gf('telemeta.models.core.CharField')(default=None, max_length=250, null=True, blank=True)),
            ('url', self.gf('telemeta.models.core.CharField')(default='', max_length=500, blank=True)),
            ('credits', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('file', self.gf('telemeta.models.core.FileField')(default='', max_length=100, db_column='filename', blank=True)),
            ('collection', self.gf('telemeta.models.core.ForeignKey')(related_name='related', to=orm['telemeta.MediaCollection'])),
        ))
        db.send_create_signal('telemeta', ['MediaCollectionRelated'])

        # Adding model 'MediaItem'
        db.create_table('media_items', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('alt_title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('collector', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('collection', self.gf('telemeta.models.core.ForeignKey')(related_name='items', to=orm['telemeta.MediaCollection'])),
            ('recorded_from_date', self.gf('telemeta.models.core.DateField')(default=None, null=True, blank=True)),
            ('recorded_to_date', self.gf('telemeta.models.core.DateField')(default=None, null=True, blank=True)),
            ('location', self.gf('telemeta.models.core.WeakForeignKey')(default=None, to=orm['telemeta.Location'], null=True, blank=True)),
            ('location_comment', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('cultural_area', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('ethnic_group', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='items', null=True, blank=True, to=orm['telemeta.EthnicGroup'])),
            ('language', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('language_iso', self.gf('telemeta.models.core.ForeignKey')(default=None, related_name='items', null=True, blank=True, to=orm['telemeta.Language'])),
            ('context_comment', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('moda_execut', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('vernacular_style', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='items', null=True, blank=True, to=orm['telemeta.VernacularStyle'])),
            ('generic_style', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='items', null=True, blank=True, to=orm['telemeta.GenericStyle'])),
            ('author', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('comment', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('collector_selection', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('collector_from_collection', self.gf('telemeta.models.core.BooleanField')(default=False)),
            ('code', self.gf('telemeta.models.core.CharField')(default='', unique=True, max_length=250, blank=True)),
            ('old_code', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('track', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('creator_reference', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('external_references', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('copied_from_item', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='copies', null=True, blank=True, to=orm['telemeta.MediaItem'])),
            ('public_access', self.gf('telemeta.models.core.CharField')(default='metadata', max_length=16, blank=True)),
            ('file', self.gf('telemeta.models.core.FileField')(default='', max_length=100, db_column='filename', blank=True)),
            ('approx_duration', self.gf('telemeta.models.core.DurationField')(default='0', blank=True)),
        ))
        db.send_create_signal('telemeta', ['MediaItem'])

        # Adding model 'MediaItemRelated'
        db.create_table('media_item_related', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('date', self.gf('telemeta.models.core.DateTimeField')(default=None, auto_now=True, null=True, blank=True)),
            ('description', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('mime_type', self.gf('telemeta.models.core.CharField')(default=None, max_length=250, null=True, blank=True)),
            ('url', self.gf('telemeta.models.core.CharField')(default='', max_length=500, blank=True)),
            ('credits', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('file', self.gf('telemeta.models.core.FileField')(default='', max_length=100, db_column='filename', blank=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='related', to=orm['telemeta.MediaItem'])),
        ))
        db.send_create_signal('telemeta', ['MediaItemRelated'])

        # Adding model 'MediaItemKeyword'
        db.create_table('media_item_keywords', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='keyword_relations', to=orm['telemeta.MediaItem'])),
            ('keyword', self.gf('telemeta.models.core.ForeignKey')(related_name='item_relations', to=orm['telemeta.ContextKeyword'])),
        ))
        db.send_create_signal('telemeta', ['MediaItemKeyword'])

        # Adding unique constraint on 'MediaItemKeyword', fields ['item', 'keyword']
        db.create_unique('media_item_keywords', ['item_id', 'keyword_id'])

        # Adding model 'MediaItemPerformance'
        db.create_table('media_item_performances', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('media_item', self.gf('telemeta.models.core.ForeignKey')(related_name='performances', to=orm['telemeta.MediaItem'])),
            ('instrument', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='performances', null=True, blank=True, to=orm['telemeta.Instrument'])),
            ('alias', self.gf('telemeta.models.core.WeakForeignKey')(default=None, related_name='performances', null=True, blank=True, to=orm['telemeta.InstrumentAlias'])),
            ('instruments_num', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('musicians', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
        ))
        db.send_create_signal('telemeta', ['MediaItemPerformance'])

        # Adding model 'MediaItemAnalysis'
        db.create_table('media_analysis', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='analysis', to=orm['telemeta.MediaItem'])),
            ('analyzer_id', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('name', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('value', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('unit', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
        ))
        db.send_create_signal('telemeta', ['MediaItemAnalysis'])

        # Adding model 'MediaPart'
        db.create_table('media_parts', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='parts', to=orm['telemeta.MediaItem'])),
            ('title', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('start', self.gf('telemeta.models.core.FloatField')()),
            ('end', self.gf('telemeta.models.core.FloatField')()),
        ))
        db.send_create_signal('telemeta', ['MediaPart'])

        # Adding model 'Playlist'
        db.create_table('playlists', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('public_id', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('author', self.gf('telemeta.models.core.ForeignKey')(related_name='playlists', db_column='author', to=orm['auth.User'])),
            ('title', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('description', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
        ))
        db.send_create_signal('telemeta', ['Playlist'])

        # Adding model 'PlaylistResource'
        db.create_table('playlist_resources', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('public_id', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('playlist', self.gf('telemeta.models.core.ForeignKey')(related_name='resources', to=orm['telemeta.Playlist'])),
            ('resource_type', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('resource_id', self.gf('telemeta.models.core.CharField')(max_length=250)),
        ))
        db.send_create_signal('telemeta', ['PlaylistResource'])

        # Adding model 'MediaItemMarker'
        db.create_table('media_markers', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='markers', to=orm['telemeta.MediaItem'])),
            ('public_id', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('time', self.gf('telemeta.models.core.FloatField')(default=0, blank=True)),
            ('title', self.gf('telemeta.models.core.CharField')(default='', max_length=250, blank=True)),
            ('date', self.gf('telemeta.models.core.DateTimeField')(default=None, auto_now=True, null=True, blank=True)),
            ('description', self.gf('telemeta.models.core.TextField')(default='', blank=True)),
            ('author', self.gf('telemeta.models.core.ForeignKey')(related_name='markers', to=orm['auth.User'])),
        ))
        db.send_create_signal('telemeta', ['MediaItemMarker'])

        # Adding model 'MediaItemTranscodingFlag'
        db.create_table('media_transcoding', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('item', self.gf('telemeta.models.core.ForeignKey')(related_name='transcoding', to=orm['telemeta.MediaItem'])),
            ('mime_type', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('date', self.gf('telemeta.models.core.DateTimeField')(default=None, auto_now=True, null=True, blank=True)),
            ('value', self.gf('telemeta.models.core.BooleanField')(default=False)),
        ))
        db.send_create_signal('telemeta', ['MediaItemTranscodingFlag'])

        # Adding model 'Search'
        db.create_table('searches', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('username', self.gf('telemeta.models.core.ForeignKey')(related_name='searches', db_column='username', to=orm['auth.User'])),
            ('keywords', self.gf('telemeta.models.core.CharField')(max_length=250)),
            ('date', self.gf('telemeta.models.core.DateField')(default=None, auto_now_add=True, null=True, blank=True)),
        ))
        db.send_create_signal('telemeta', ['Search'])

    def backwards(self, orm):
        # Removing unique constraint on 'MediaItemKeyword', fields ['item', 'keyword']
        db.delete_unique('media_item_keywords', ['item_id', 'keyword_id'])

        # Removing unique constraint on 'MediaCorpusCollectionRelation', fields ['collection', 'corpus']
        db.delete_unique('media_corpus_collection_relations', ['collection_id', 'corpus_id'])

        # Removing unique constraint on 'InstrumentAliasRelation', fields ['alias', 'instrument']
        db.delete_unique('instrument_alias_relations', ['alias_id', 'instrument_id'])

        # Removing unique constraint on 'InstrumentRelation', fields ['instrument', 'parent_instrument']
        db.delete_unique('instrument_relations', ['instrument_id', 'parent_instrument_id'])

        # Removing unique constraint on 'LocationRelation', fields ['location', 'ancestor_location']
        db.delete_unique('location_relations', ['location_id', 'ancestor_location_id'])

        # Removing unique constraint on 'LocationAlias', fields ['location', 'alias']
        db.delete_unique('location_aliases', ['location_id', 'alias'])

        # Removing unique constraint on 'EthnicGroupAlias', fields ['ethnic_group', 'value']
        db.delete_unique('ethnic_group_aliases', ['ethnic_group_id', 'value'])

        # Deleting model 'PhysicalFormat'
        db.delete_table('physical_formats')

        # Deleting model 'PublishingStatus'
        db.delete_table('publishing_status')

        # Deleting model 'AcquisitionMode'
        db.delete_table('acquisition_modes')

        # Deleting model 'MetadataAuthor'
        db.delete_table('metadata_authors')

        # Deleting model 'MetadataWriter'
        db.delete_table('metadata_writers')

        # Deleting model 'LegalRight'
        db.delete_table('legal_rights')

        # Deleting model 'RecordingContext'
        db.delete_table('recording_contexts')

        # Deleting model 'AdConversion'
        db.delete_table('ad_conversions')

        # Deleting model 'VernacularStyle'
        db.delete_table('vernacular_styles')

        # Deleting model 'GenericStyle'
        db.delete_table('generic_styles')

        # Deleting model 'ContextKeyword'
        db.delete_table('context_keywords')

        # Deleting model 'Publisher'
        db.delete_table('publishers')

        # Deleting model 'PublisherCollection'
        db.delete_table('publisher_collections')

        # Deleting model 'EthnicGroup'
        db.delete_table('ethnic_groups')

        # Deleting model 'EthnicGroupAlias'
        db.delete_table('ethnic_group_aliases')

        # Deleting model 'Location'
        db.delete_table('locations')

        # Deleting model 'LocationType'
        db.delete_table('location_types')

        # Deleting model 'LocationAlias'
        db.delete_table('location_aliases')

        # Deleting model 'LocationRelation'
        db.delete_table('location_relations')

        # Deleting model 'Revision'
        db.delete_table('revisions')

        # Deleting model 'UserProfile'
        db.delete_table('profiles')

        # Deleting model 'Instrument'
        db.delete_table('instruments')

        # Deleting model 'InstrumentAlias'
        db.delete_table('instrument_aliases')

        # Deleting model 'InstrumentRelation'
        db.delete_table('instrument_relations')

        # Deleting model 'InstrumentAliasRelation'
        db.delete_table('instrument_alias_relations')

        # Deleting model 'Language'
        db.delete_table('languages')

        # Deleting model 'MediaCorpus'
        db.delete_table('media_corpus')

        # Deleting model 'MediaCorpusCollectionRelation'
        db.delete_table('media_corpus_collection_relations')

        # Deleting model 'MediaCollection'
        db.delete_table('media_collections')

        # Deleting model 'MediaCollectionRelated'
        db.delete_table('media_collection_related')

        # Deleting model 'MediaItem'
        db.delete_table('media_items')

        # Deleting model 'MediaItemRelated'
        db.delete_table('media_item_related')

        # Deleting model 'MediaItemKeyword'
        db.delete_table('media_item_keywords')

        # Deleting model 'MediaItemPerformance'
        db.delete_table('media_item_performances')

        # Deleting model 'MediaItemAnalysis'
        db.delete_table('media_analysis')

        # Deleting model 'MediaPart'
        db.delete_table('media_parts')

        # Deleting model 'Playlist'
        db.delete_table('playlists')

        # Deleting model 'PlaylistResource'
        db.delete_table('playlist_resources')

        # Deleting model 'MediaItemMarker'
        db.delete_table('media_markers')

        # Deleting model 'MediaItemTranscodingFlag'
        db.delete_table('media_transcoding')

        # Deleting model 'Search'
        db.delete_table('searches')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'telemeta.acquisitionmode': {
            'Meta': {'ordering': "['value']", 'object_name': 'AcquisitionMode', 'db_table': "'acquisition_modes'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.adconversion': {
            'Meta': {'ordering': "['value']", 'object_name': 'AdConversion', 'db_table': "'ad_conversions'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.contextkeyword': {
            'Meta': {'ordering': "['value']", 'object_name': 'ContextKeyword', 'db_table': "'context_keywords'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.ethnicgroup': {
            'Meta': {'ordering': "['value']", 'object_name': 'EthnicGroup', 'db_table': "'ethnic_groups'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.ethnicgroupalias': {
            'Meta': {'ordering': "['ethnic_group__value']", 'unique_together': "(('ethnic_group', 'value'),)", 'object_name': 'EthnicGroupAlias', 'db_table': "'ethnic_group_aliases'"},
            'ethnic_group': ('telemeta.models.core.ForeignKey', [], {'related_name': "'aliases'", 'to': "orm['telemeta.EthnicGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.genericstyle': {
            'Meta': {'ordering': "['value']", 'object_name': 'GenericStyle', 'db_table': "'generic_styles'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.instrument': {
            'Meta': {'object_name': 'Instrument', 'db_table': "'instruments'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.instrumentalias': {
            'Meta': {'object_name': 'InstrumentAlias', 'db_table': "'instrument_aliases'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.instrumentaliasrelation': {
            'Meta': {'unique_together': "(('alias', 'instrument'),)", 'object_name': 'InstrumentAliasRelation', 'db_table': "'instrument_alias_relations'"},
            'alias': ('telemeta.models.core.ForeignKey', [], {'related_name': "'other_name'", 'to': "orm['telemeta.InstrumentAlias']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instrument': ('telemeta.models.core.ForeignKey', [], {'related_name': "'relation'", 'to': "orm['telemeta.InstrumentAlias']"})
        },
        'telemeta.instrumentrelation': {
            'Meta': {'unique_together': "(('instrument', 'parent_instrument'),)", 'object_name': 'InstrumentRelation', 'db_table': "'instrument_relations'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instrument': ('telemeta.models.core.ForeignKey', [], {'related_name': "'parent_relation'", 'to': "orm['telemeta.Instrument']"}),
            'parent_instrument': ('telemeta.models.core.ForeignKey', [], {'related_name': "'child_relation'", 'to': "orm['telemeta.Instrument']"})
        },
        'telemeta.language': {
            'Meta': {'ordering': "['name']", 'object_name': 'Language', 'db_table': "'languages'"},
            'comment': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '3', 'blank': 'True'}),
            'name': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'part1': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '1', 'blank': 'True'}),
            'part2B': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '3', 'blank': 'True'}),
            'part2T': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '3', 'blank': 'True'}),
            'scope': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '1', 'blank': 'True'}),
            'type': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '1', 'blank': 'True'})
        },
        'telemeta.legalright': {
            'Meta': {'ordering': "['value']", 'object_name': 'LegalRight', 'db_table': "'legal_rights'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.location': {
            'Meta': {'ordering': "['name']", 'object_name': 'Location', 'db_table': "'locations'"},
            'complete_type': ('telemeta.models.core.ForeignKey', [], {'related_name': "'locations'", 'to': "orm['telemeta.LocationType']"}),
            'current_location': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'past_names'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.Location']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_authoritative': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'latitude': ('telemeta.models.core.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'longitude': ('telemeta.models.core.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'name': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '150'}),
            'type': ('telemeta.models.core.IntegerField', [], {'default': '0', 'db_index': 'True', 'blank': 'True'})
        },
        'telemeta.locationalias': {
            'Meta': {'ordering': "['alias']", 'unique_together': "(('location', 'alias'),)", 'object_name': 'LocationAlias', 'db_table': "'location_aliases'"},
            'alias': ('telemeta.models.core.CharField', [], {'max_length': '150'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_authoritative': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'location': ('telemeta.models.core.ForeignKey', [], {'related_name': "'aliases'", 'to': "orm['telemeta.Location']"})
        },
        'telemeta.locationrelation': {
            'Meta': {'ordering': "['ancestor_location__name']", 'unique_together': "(('location', 'ancestor_location'),)", 'object_name': 'LocationRelation', 'db_table': "'location_relations'"},
            'ancestor_location': ('telemeta.models.core.ForeignKey', [], {'related_name': "'descendant_relations'", 'to': "orm['telemeta.Location']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_authoritative': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'is_direct': ('telemeta.models.core.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'location': ('telemeta.models.core.ForeignKey', [], {'related_name': "'ancestor_relations'", 'to': "orm['telemeta.Location']"})
        },
        'telemeta.locationtype': {
            'Meta': {'ordering': "['name']", 'object_name': 'LocationType', 'db_table': "'location_types'"},
            'code': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('telemeta.models.core.CharField', [], {'max_length': '150'})
        },
        'telemeta.mediacollection': {
            'Meta': {'ordering': "['code']", 'object_name': 'MediaCollection', 'db_table': "'media_collections'"},
            'a_informer_07_03': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'acquisition_mode': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.AcquisitionMode']"}),
            'ad_conversion': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.AdConversion']"}),
            'alt_ids': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'alt_title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'approx_duration': ('telemeta.models.core.DurationField', [], {'default': "'0'", 'blank': 'True'}),
            'booklet_author': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'booklet_description': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'cnrs_contributor': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'code': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'}),
            'collector': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'collector_is_creator': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'comment': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'conservation_site': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'creator': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'doctype_code': ('telemeta.models.core.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'external_references': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'items_done': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'legal_rights': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.LegalRight']"}),
            'metadata_author': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.MetadataAuthor']"}),
            'metadata_writer': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.MetadataWriter']"}),
            'old_code': ('telemeta.models.core.CharField', [], {'default': 'None', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'physical_format': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.PhysicalFormat']"}),
            'physical_items_num': ('telemeta.models.core.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'public_access': ('telemeta.models.core.CharField', [], {'default': "'metadata'", 'max_length': '16', 'blank': 'True'}),
            'publisher': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.Publisher']"}),
            'publisher_collection': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.PublisherCollection']"}),
            'publisher_serial': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'publishing_status': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.PublishingStatus']"}),
            'recorded_from_year': ('telemeta.models.core.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'recorded_to_year': ('telemeta.models.core.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'recording_context': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'collections'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.RecordingContext']"}),
            'reference': ('telemeta.models.core.CharField', [], {'default': 'None', 'max_length': '250', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'state': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'travail': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'year_published': ('telemeta.models.core.IntegerField', [], {'default': '0', 'blank': 'True'})
        },
        'telemeta.mediacollectionrelated': {
            'Meta': {'object_name': 'MediaCollectionRelated', 'db_table': "'media_collection_related'"},
            'collection': ('telemeta.models.core.ForeignKey', [], {'related_name': "'related'", 'to': "orm['telemeta.MediaCollection']"}),
            'credits': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'date': ('telemeta.models.core.DateTimeField', [], {'default': 'None', 'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'description': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'file': ('telemeta.models.core.FileField', [], {'default': "''", 'max_length': '100', 'db_column': "'filename'", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_type': ('telemeta.models.core.CharField', [], {'default': 'None', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'url': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '500', 'blank': 'True'})
        },
        'telemeta.mediacorpus': {
            'Meta': {'ordering': "['code']", 'object_name': 'MediaCorpus', 'db_table': "'media_corpus'"},
            'code': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'}),
            'description': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'public_access': ('telemeta.models.core.CharField', [], {'default': "'metadata'", 'max_length': '16', 'blank': 'True'}),
            'reference': ('telemeta.models.core.CharField', [], {'default': 'None', 'max_length': '250', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.mediacorpuscollectionrelation': {
            'Meta': {'unique_together': "(('collection', 'corpus'),)", 'object_name': 'MediaCorpusCollectionRelation', 'db_table': "'media_corpus_collection_relations'"},
            'collection': ('telemeta.models.core.ForeignKey', [], {'related_name': "'parent_relation'", 'to': "orm['telemeta.MediaCollection']"}),
            'corpus': ('telemeta.models.core.ForeignKey', [], {'related_name': "'child_relation'", 'to': "orm['telemeta.MediaCorpus']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'telemeta.mediaitem': {
            'Meta': {'object_name': 'MediaItem', 'db_table': "'media_items'"},
            'alt_title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'approx_duration': ('telemeta.models.core.DurationField', [], {'default': "'0'", 'blank': 'True'}),
            'author': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'code': ('telemeta.models.core.CharField', [], {'default': "''", 'unique': 'True', 'max_length': '250', 'blank': 'True'}),
            'collection': ('telemeta.models.core.ForeignKey', [], {'related_name': "'items'", 'to': "orm['telemeta.MediaCollection']"}),
            'collector': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'collector_from_collection': ('telemeta.models.core.BooleanField', [], {'default': 'False'}),
            'collector_selection': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'comment': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'context_comment': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'copied_from_item': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'copies'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.MediaItem']"}),
            'creator_reference': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'cultural_area': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'ethnic_group': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'items'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.EthnicGroup']"}),
            'external_references': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'file': ('telemeta.models.core.FileField', [], {'default': "''", 'max_length': '100', 'db_column': "'filename'", 'blank': 'True'}),
            'generic_style': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'items'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.GenericStyle']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'language_iso': ('telemeta.models.core.ForeignKey', [], {'default': 'None', 'related_name': "'items'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.Language']"}),
            'location': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'to': "orm['telemeta.Location']", 'null': 'True', 'blank': 'True'}),
            'location_comment': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'moda_execut': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'old_code': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'public_access': ('telemeta.models.core.CharField', [], {'default': "'metadata'", 'max_length': '16', 'blank': 'True'}),
            'recorded_from_date': ('telemeta.models.core.DateField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'recorded_to_date': ('telemeta.models.core.DateField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'track': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'vernacular_style': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'items'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.VernacularStyle']"})
        },
        'telemeta.mediaitemanalysis': {
            'Meta': {'ordering': "['name']", 'object_name': 'MediaItemAnalysis', 'db_table': "'media_analysis'"},
            'analyzer_id': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'analysis'", 'to': "orm['telemeta.MediaItem']"}),
            'name': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'unit': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'})
        },
        'telemeta.mediaitemkeyword': {
            'Meta': {'unique_together': "(('item', 'keyword'),)", 'object_name': 'MediaItemKeyword', 'db_table': "'media_item_keywords'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'keyword_relations'", 'to': "orm['telemeta.MediaItem']"}),
            'keyword': ('telemeta.models.core.ForeignKey', [], {'related_name': "'item_relations'", 'to': "orm['telemeta.ContextKeyword']"})
        },
        'telemeta.mediaitemmarker': {
            'Meta': {'object_name': 'MediaItemMarker', 'db_table': "'media_markers'"},
            'author': ('telemeta.models.core.ForeignKey', [], {'related_name': "'markers'", 'to': "orm['auth.User']"}),
            'date': ('telemeta.models.core.DateTimeField', [], {'default': 'None', 'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'description': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'markers'", 'to': "orm['telemeta.MediaItem']"}),
            'public_id': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'time': ('telemeta.models.core.FloatField', [], {'default': '0', 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'})
        },
        'telemeta.mediaitemperformance': {
            'Meta': {'object_name': 'MediaItemPerformance', 'db_table': "'media_item_performances'"},
            'alias': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'performances'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.InstrumentAlias']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instrument': ('telemeta.models.core.WeakForeignKey', [], {'default': 'None', 'related_name': "'performances'", 'null': 'True', 'blank': 'True', 'to': "orm['telemeta.Instrument']"}),
            'instruments_num': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'media_item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'performances'", 'to': "orm['telemeta.MediaItem']"}),
            'musicians': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'})
        },
        'telemeta.mediaitemrelated': {
            'Meta': {'object_name': 'MediaItemRelated', 'db_table': "'media_item_related'"},
            'credits': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'date': ('telemeta.models.core.DateTimeField', [], {'default': 'None', 'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'description': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'file': ('telemeta.models.core.FileField', [], {'default': "''", 'max_length': '100', 'db_column': "'filename'", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'related'", 'to': "orm['telemeta.MediaItem']"}),
            'mime_type': ('telemeta.models.core.CharField', [], {'default': 'None', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'title': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'url': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '500', 'blank': 'True'})
        },
        'telemeta.mediaitemtranscodingflag': {
            'Meta': {'object_name': 'MediaItemTranscodingFlag', 'db_table': "'media_transcoding'"},
            'date': ('telemeta.models.core.DateTimeField', [], {'default': 'None', 'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'transcoding'", 'to': "orm['telemeta.MediaItem']"}),
            'mime_type': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'value': ('telemeta.models.core.BooleanField', [], {'default': 'False'})
        },
        'telemeta.mediapart': {
            'Meta': {'object_name': 'MediaPart', 'db_table': "'media_parts'"},
            'end': ('telemeta.models.core.FloatField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('telemeta.models.core.ForeignKey', [], {'related_name': "'parts'", 'to': "orm['telemeta.MediaItem']"}),
            'start': ('telemeta.models.core.FloatField', [], {}),
            'title': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.metadataauthor': {
            'Meta': {'ordering': "['value']", 'object_name': 'MetadataAuthor', 'db_table': "'metadata_authors'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.metadatawriter': {
            'Meta': {'ordering': "['value']", 'object_name': 'MetadataWriter', 'db_table': "'metadata_writers'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.physicalformat': {
            'Meta': {'ordering': "['value']", 'object_name': 'PhysicalFormat', 'db_table': "'physical_formats'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.playlist': {
            'Meta': {'object_name': 'Playlist', 'db_table': "'playlists'"},
            'author': ('telemeta.models.core.ForeignKey', [], {'related_name': "'playlists'", 'db_column': "'author'", 'to': "orm['auth.User']"}),
            'description': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'public_id': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'title': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.playlistresource': {
            'Meta': {'object_name': 'PlaylistResource', 'db_table': "'playlist_resources'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'playlist': ('telemeta.models.core.ForeignKey', [], {'related_name': "'resources'", 'to': "orm['telemeta.Playlist']"}),
            'public_id': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'resource_id': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'resource_type': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.publisher': {
            'Meta': {'ordering': "['value']", 'object_name': 'Publisher', 'db_table': "'publishers'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.publishercollection': {
            'Meta': {'ordering': "['value']", 'object_name': 'PublisherCollection', 'db_table': "'publisher_collections'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'publisher': ('telemeta.models.core.ForeignKey', [], {'related_name': "'publisher_collections'", 'to': "orm['telemeta.Publisher']"}),
            'value': ('telemeta.models.core.CharField', [], {'max_length': '250'})
        },
        'telemeta.publishingstatus': {
            'Meta': {'ordering': "['value']", 'object_name': 'PublishingStatus', 'db_table': "'publishing_status'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.recordingcontext': {
            'Meta': {'ordering': "['value']", 'object_name': 'RecordingContext', 'db_table': "'recording_contexts'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        },
        'telemeta.revision': {
            'Meta': {'object_name': 'Revision', 'db_table': "'revisions'"},
            'change_type': ('telemeta.models.core.CharField', [], {'max_length': '16'}),
            'element_id': ('telemeta.models.core.IntegerField', [], {}),
            'element_type': ('telemeta.models.core.CharField', [], {'max_length': '16'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('telemeta.models.core.DateTimeField', [], {'default': 'None', 'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'user': ('telemeta.models.core.ForeignKey', [], {'related_name': "'revisions'", 'db_column': "'username'", 'to': "orm['auth.User']"})
        },
        'telemeta.search': {
            'Meta': {'object_name': 'Search', 'db_table': "'searches'"},
            'date': ('telemeta.models.core.DateField', [], {'default': 'None', 'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('telemeta.models.core.CharField', [], {'max_length': '250'}),
            'username': ('telemeta.models.core.ForeignKey', [], {'related_name': "'searches'", 'db_column': "'username'", 'to': "orm['auth.User']"})
        },
        'telemeta.userprofile': {
            'Meta': {'object_name': 'UserProfile', 'db_table': "'profiles'"},
            'address': ('telemeta.models.core.TextField', [], {'default': "''", 'blank': 'True'}),
            'expiration_date': ('telemeta.models.core.DateField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'function': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'institution': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'telephone': ('telemeta.models.core.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'user': ('telemeta.models.core.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'})
        },
        'telemeta.vernacularstyle': {
            'Meta': {'ordering': "['value']", 'object_name': 'VernacularStyle', 'db_table': "'vernacular_styles'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('telemeta.models.core.CharField', [], {'unique': 'True', 'max_length': '250'})
        }
    }

    complete_apps = ['telemeta']



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2015 Parisson SARL
#
# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from __future__ import division

__all__ = ['DurationField', 'Duration', 'WeakForeignKey',
           'CharField', 'TextField', 'IntegerField', 'BooleanField',
           'DateTimeField', 'FileField', 'ForeignKey', 'FloatField', 'DateField',
           'RequiredFieldError',]

import datetime, re
from django import forms
from django.db import models
from django.utils.translation import ugettext_lazy as _
from south.modelsinspector import add_introspection_rules


class Duration(object):
    """Represent a time duration"""
    def __init__(self, *args, **kwargs):
        if len(args) and isinstance(args[0], datetime.timedelta):
            self._delta = datetime.timedelta(days=args[0].days, seconds=args[0].seconds)
        else:
            self._delta = datetime.timedelta(*args, **kwargs)

    def __decorate(self, method, other):
        if isinstance(other, Duration):
            res = method(other._delta)
        else:
            res = method(other)
        if type(res) == datetime.timedelta:
            return Duration(res)

        return res

    def __add__(self, other):
        return self.__decorate(self._delta.__add__, other)

    def __nonzero__(self):
        return self._delta.__nonzero__()

    def __str__(self):
        hours   = self._delta.days * 24 + self._delta.seconds / 3600
        minutes = (self._delta.seconds % 3600) / 60
        seconds = self._delta.seconds % 60

        return "%.2d:%.2d:%.2d" % (hours, minutes, seconds)

    def __unicode__(self):
        return self.__str__()

    @staticmethod
    def fromstr(str):
        if not str:
            return Duration()

        test = re.match('^([0-9]+)(?::([0-9]+)(?::([0-9]+))?)?$', str)
        if test:
            groups = test.groups()
            try:
                hours = minutes = seconds = 0
                if groups[0]:
                    hours = int(groups[0])
                    if groups[1]:
                        minutes = int(groups[1])
                        if groups[2]:
                            seconds = int(groups[2])

                return Duration(hours=hours, minutes=minutes, seconds=seconds)
            except TypeError:
                print groups
                raise
        else:
            raise ValueError("Malformed duration string: " + str)

    def as_seconds(self):
        return self._delta.days * 24 * 3600 + self._delta.seconds


def normalize_field(args, default_value=None):
    """Normalize field constructor arguments, so that the field is marked blank=True
       and has a default value by default.

       This behaviour can be disabled by passing the special argument required=True.

       The default value can also be overriden with the default=value argument.
       """
    required = False
    if args.has_key('required'):
        required = args['required']
        args.pop('required')

    args['blank'] = not required

    if not required:
        if not args.has_key('default'):
            if args.get('null'):
                args['default'] = None
            elif default_value is not None:
                args['default'] = default_value

    return args


class DurationField(models.Field):
    """Duration Django model field based on Django TimeField.
    Essentially the same as a TimeField, but with values over 24h allowed.

    The constructor arguments are also normalized with normalize_field().
    """

    description = _("Duration")

    __metaclass__ = models.SubfieldBase

    default_error_messages = {
        'invalid': _('Enter a valid duration in HH:MM[:ss] format.'),
    }

    def __init__(self, *args, **kwargs):
        super(DurationField, self).__init__(*args, **normalize_field(kwargs, '0'))

    def db_type(self):
        return 'int'

    def to_python(self, value):
        if value is None:
            return None
        if isinstance(value, int) or isinstance(value, long):
            return Duration(seconds=value)
        if isinstance(value, datetime.time):
            return Duration(hours=value.hour, minutes=value.minute, seconds=value.second)
        if isinstance(value, datetime.datetime):
            # Not usually a good idea to pass in a datetime here (it loses
            # information), but this can be a side-effect of interacting with a
            # database backend (e.g. Oracle), so we'll be accommodating.
            return self.to_python(value.time())
        else:
            value = str(value)
        try:
            return Duration.fromstr(value)
        except ValueError:
            raise exceptions.ValidationError(self.error_messages['invalid'])

    def get_prep_value(self, value):
        return self.to_python(value)

    def get_db_prep_value(self, value, connection=None, prepared=False):
        # Casts times into the format expected by the backend
        try:
            return value.as_seconds()
        except:
            return value

    def value_to_string(self, obj):
        val = self._get_val_from_obj(obj)
        if val is None:
            data = ''
        else:
            data = unicode(val)
        return data

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.CharField}
        defaults.update(kwargs)
        return super(DurationField, self).formfield(**defaults)


class ForeignKey(models.ForeignKey):
    """The constructor arguments of this ForeignKey are normalized
    with normalize_field(), however the field is marked required by default
    unless it is allowed to be null."""

    def __init__(self, to, **kwargs):
        if not kwargs.has_key('required'):
            if not kwargs.get('null'):
                kwargs['required'] = True

        super(ForeignKey, self).__init__(to, **normalize_field(kwargs, 0))


class WeakForeignKey(ForeignKey):
    """A weak foreign key is the same as foreign key but without cascading
    delete. Instead the reference is set to null when the referenced record
    get deleted. This emulates the ON DELETE SET NULL sql behaviour.

    This field is automatically allowed to be null, there's no need to pass
    null=True.

    The constructor arguments are normalized with normalize_field() by the
    parent ForeignKey

    Warning: must be used in conjunction with EnhancedQuerySet, EnhancedManager,
    and EnhancedModel
    """
    def __init__(self, to, **kwargs):
        kwargs['null'] = True
        super(WeakForeignKey, self).__init__(to, **kwargs)



class CharField(models.CharField):
    """This is a CharField with a default max_length of 250.

       The arguments are also normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('max_length'):
            kwargs['max_length'] = 250
        super(CharField, self).__init__(*args, **normalize_field(kwargs, ''))


class IntegerField(models.IntegerField):
    """IntegerField normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        super(IntegerField, self).__init__(*args, **normalize_field(kwargs, 0))


class BooleanField(models.BooleanField):
    """BooleanField normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        super(BooleanField, self).__init__(*args, **normalize_field(kwargs, False))


class TextField(models.TextField):
    """TextField normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        super(TextField, self).__init__(*args, **normalize_field(kwargs, ''))


class DateTimeField(models.DateTimeField):
    """DateTimeField normalized with normalize_field(). This field is allowed to
    be null by default unless null=False is passed"""

    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('null'):
            kwargs['null'] = True
        super(DateTimeField, self).__init__(*args, **normalize_field(kwargs))


class FileField(models.FileField):
    """FileField normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        super(FileField, self).__init__(*args, **normalize_field(kwargs, ''))


class FloatField(models.FloatField):
    """FloatField normalized with normalize_field()"""

    def __init__(self, *args, **kwargs):
        super(FloatField, self).__init__(*args, **normalize_field(kwargs, 0))


class DateField(models.DateField):
    """DateField normalized with normalize_field(). This field is allowed to
    be null by default unless null=False is passed"""

    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('null'):
            kwargs['null'] = True
        super(DateField, self).__init__(*args, **normalize_field(kwargs))


class RequiredFieldError(Exception):
    def __init__(self, model, field):
        self.model = model
        self.field = field
        super(Exception, self).__init__('%s.%s is required' % (model._meta.object_name, field.name))



# South introspection rules
add_introspection_rules([], ["^telemeta\.models\.fields\.CharField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.TextField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.FileField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.IntegerField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.BooleanField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.DateTimeField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.DateField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.FloatField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.DurationField"])
add_introspection_rules([], ["^telemeta\.models\.fields\.ForeignKey"])
add_introspection_rules([], ["^telemeta\.models\.fields\.WeakForeignKey"])




# -*- coding: utf-8 -*-
# Copyright (C) 2010 Samalyse SARL
# Copyright (C) 2010-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from django.utils.translation import ugettext_lazy as _
from telemeta.models.core import *
from telemeta.models.system import *


class MediaResource(ModelCore):
    "Base class of all media objects"

    def public_access_label(self):
        if self.public_access == 'metadata':
            return _('Metadata only')
        elif self.public_access == 'full':
            return _('Sound and metadata')

        return _('Private data')
    public_access_label.verbose_name = _('access type')

    def set_revision(self, user):
        "Save a media object and add a revision"
        Revision.touch(self, user)

    def get_revision(self):
        revisions = Revision.objects.filter(element_type=self.element_type, element_id=self.id).order_by('-time')
        if revisions:
            return revisions[0]
        else:
            return None

    class Meta:
        abstract = True


class MediaBaseResource(MediaResource):
    "Describe a media base resource"

    title                 = CharField(_('title'), required=True)
    description           = CharField(_('description_old'))
    descriptions          = TextField(_('description'))
    code                  = CharField(_('code'), unique=True, required=True)
    public_access         = CharField(_('public access'), choices=PUBLIC_ACCESS_CHOICES, max_length=16, default="metadata")

    def __unicode__(self):
        return self.code

    @property
    def public_id(self):
        return self.code

    def save(self, *args, **kwargs):
        super(MediaBaseResource, self).save(*args, **kwargs)

    def get_fields(self):
        return self._meta.fields

    class Meta(MetaCore):
        abstract = True


class MediaRelated(MediaResource):
    "Related media"

    element_type = 'media'

    title           = CharField(_('title'))
    date            = DateTimeField(_('date'), auto_now=True)
    description     = TextField(_('description'))
    mime_type       = CharField(_('mime_type'))
    url             = CharField(_('url'), max_length=500)
    credits         = CharField(_('credits'))
    file            = FileField(_('file'), upload_to='items/%Y/%m/%d', db_column="filename", max_length=255)

    def is_image(self):
        is_url_image = False
        if self.url:
            url_types = ['.png', '.jpg', '.gif', '.jpeg']
            for type in url_types:
                if type in self.url or type.upper() in self.url:
                    is_url_image = True
        return 'image' in self.mime_type or is_url_image

    def save(self, *args, **kwargs):
        super(MediaRelated, self).save(*args, **kwargs)

    def set_mime_type(self):
        if self.file:
            self.mime_type = mimetypes.guess_type(self.file.path)[0]

    def is_kdenlive_session(self):
        if self.file:
            return '.kdenlive' in self.file.path
        else:
            return False

    def __unicode__(self):
        if self.title and not re.match('^ *N *$', self.title):
            return self.title
        elif self.file:
            return unicode(self.file.path.split(os.sep)[-1])
        elif self.url:
            return unicode(self.url.split('/')[-1])
        else:
            return '_'

    class Meta:
        abstract = True





# -*- coding: utf-8 -*-
# Generic OAI-PMH Data Provider module
#
# Copyright (C) 2009 Samalyse SARL
# Author: Olivier Guilyardi <olivier samalyse com>
#
# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
import time
try:
  import libxml2dom as dom
except ImportError:
    import xml.dom.minidom as dom
    try:
        import libxml2
    except ImportError:
        # dangerous: minidom output formatting isn't very good, libxml2 is much better
        pass

class IDataSource(object):
    """Interface for OAI datasource adapters"""

    def get_earliest_time(self):
        """Must return the change time of the oldest record(s) as a datetime object"""
        pass

    def get_record(self, id):
        """Must return a tuple of the form:
              (
                [(dublin core element, element value), ...],
                change time
              )
           or None if the record doesn't exist. In case the id isn't wellformed
           a BadArgumentError should be raised.

           The dublin core data must contain an 'identifier' element, which is the same
           as the id parameter."""
        pass

    def count_records(self, from_time = None, until_time = None):
        """Must return the number of records between (optional) from and until change time."""
        pass

    def list_records(self, offset, limit, from_time = None, until_time = None):
        """Must return the list of records between (optional) from and
           until change time, starting from record at offset, with a maximum of limit
           entries. Each entry of the list must be a tuple of the same form as returned
           by getRecord().

           If no record matches, should return an empty list. The dublin core data must
           contain an 'identifier' element, which can be used as a parameter to get_record()."""
        pass

def iso_time(date_time = None):
    """Encode a datetime object using ISO8601 format"""
    if not date_time:
        date_time = datetime.now()
    return date_time.strftime('%Y-%m-%dT%H:%M:%SZ')

def parse_iso_time(str):
    """Parse an ISO8601 time string into a datetime object, or return None on failure"""
    # Avoid datetime.strptime() for compatibility with python < 2.5
    try:
        s = time.strptime(str, '%Y-%m-%dT%H:%M:%SZ')
        return datetime(s.tm_year, s.tm_mon, s.tm_mday, s.tm_hour, s.tm_min, s.tm_sec)
    except ValueError:
        try:
            s = time.strptime(str, '%Y-%m-%d')
            return datetime(s.tm_year, s.tm_mon, s.tm_mday)
        except ValueError:
            return None

def doc_to_string(doc):
    if dom.__name__ == 'libxml2dom':
        return doc.toString(encoding='utf-8', prettyprint=True).decode('utf-8')
    try:
        doc2 = libxml2.parseDoc(doc.toxml(encoding="utf-8"))
        xml = unicode(doc2.serialize(encoding="utf-8", format=1), "utf-8")
        doc2.free()
        return xml
    except NameError:
        return doc.toprettyxml(encoding="utf-8")

class ArgumentValidator(object):
    """OAI-PMH request argument validator"""

    def __init__(self, request, response):
        self.response = response
        self.opt_args = []
        self.required_args = ['verb']
        self.request = request
        self.format = None

    def optional(self, *args):
        """Add optional arguments"""
        self.opt_args.extend(args)

    def require(self, *args):
        """Add required arguments"""
        self.required_args.extend(args)

    def accept_format(self, format):
        """Indicate which metadata format is supported"""
        self.format = format

    def has_verb(self):
        """Check if the request includes a valid Verb, return True if it does, False otherwise,
           setting an error into the response"""

        valid = ['GetRecord', 'Identify', 'ListIdentifiers', 'ListMetadataFormats', 'ListRecords', 'ListSets']

        result = False
        if self.request.has_key('verb'):
            try:
                valid.index(self.request['verb'])
                result = True
            except ValueError:
                pass

        if not result:
            self.response.error('badVerb')

        return result

    def validate(self):
        """Perform validation, return True if successfull, False otherwise, setting appropriate
           errors into the response"""
        all_args    = []
        all_args[:] = self.opt_args[:]
        all_args.extend(self.required_args)
        for k in self.request:
            try:
                all_args.index(k)
                if (k == 'set'):
                    self.response.error('noSetHierarchy')
                    return False
            except ValueError:
                self.response.error('badArgument', 'Invalid argument: %s' % k)
                return False

        return self.pre_validate()

    def pre_validate(self):
        """Same as validate(), but doesn't not check for unknown arguments"""

        for k in self.required_args:
            if not self.request.has_key(k):
                self.response.error('badArgument', 'Missing required argument: %s' % k)
                return False

        for k in self.request:
            if k == 'metadataPrefix':
                if self.format:
                    if self.format != self.request[k]:
                        self.response.error('cannotDisseminateFormat')
                        return False
                else:
                    raise Exception('Can\'t validate metadataPrefix argument: supported format isn\'t defined')
            elif (k == 'from') or (k == 'until'):
                if not parse_iso_time(self.request[k]):
                    self.response.error('badArgument', "Invalid ISO8601 time format in '%s' argument" % k)
                    return False

        return True

class DataProvider(object):
    """OAI-PMH Data Provider"""

    max_records_per_response = 500

    def __init__(self, datasource, repository_name, base_url, admin_email):
        self.datasource = datasource
        self.identity = [
            ('repositoryName',   repository_name),
            ('baseURL',          base_url),
            ('protocolVersion',  '2.0'),
            ('adminEmail',       admin_email),
            ('deletedRecord',    'no'),
            ('granularity',      'YYYY-MM-DDThh:mm:ssZ')
        ]

    def parse_time_range(self, args):
        if args.get('from'):
            from_time = parse_iso_time(args['from'])
        else:
            from_time = None
        if args.get('until'):
            until_time = parse_iso_time(args['until'])
        else:
            until_time = None

        return from_time, until_time

    def handle(self, args):
        """Handle a request and return the response as a DOM document"""

        response = Response(self.identity, self.datasource)
        response.max_records_per_response = self.max_records_per_response

        validator = ArgumentValidator(args, response)
        validator.accept_format('oai_dc')

        if validator.has_verb():

            verb = args['verb']
            response.set_verb(verb)

            if verb == 'Identify':
                validator.validate() and response.identify()
            elif verb == 'GetRecord':
                validator.require('identifier', 'metadataPrefix')
                validator.validate() and response.get_record(args['identifier'])
            elif verb == 'ListIdentifiers' or verb == 'ListRecords':
                validator.optional('metadataPrefix', 'from', 'until', 'set', 'resumptionToken')
                from_time, until_time = self.parse_time_range(args)
                token = args.get('resumptionToken')
                if validator.validate():
                    response.list_records(from_time, until_time, token, ids_only = (verb == 'ListIdentifiers'))
            elif verb == 'ListSets':
                validator.optional('resumptionToken')
                validator.validate() and response.error('noSetHierarchy')
            elif verb == 'ListMetadataFormats':
                validator.optional('identifier')
                validator.validate() and response.list_formats(args.get('identifier'))

        xml = doc_to_string(response.doc)
        response.free()
        return xml

class Response(object):
    """OAI-PMH response generation"""

    max_records_per_response = 500

    def __init__(self, identity, datasource):
        self.identity = identity
        self.datasource = datasource

        impl = dom.getDOMImplementation()
        self.doc = impl.createDocument(None, 'OAI-PMH', None)
        self.root = self.doc.firstChild
        self.root.setAttribute('xmlns', 'http://www.openarchives.org/OAI/2.0/')
        self.root.setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
        self.root.setAttribute('xsi:schemaLocation', 'http://www.openarchives.org/OAI/2.0/ '
                                                     'http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd')
        self.append_elements(self.root, {'responseDate': iso_time()})
        self.request = self.root.appendChild(self.doc.createElement('request'))
        for k, v in self.identity:
            if k == 'baseURL':
                url = v
                break
        self.request.appendChild(self.doc.createTextNode(url))

    def append_elements(self, parent, elements, prefix=None):
        """Append several elements to parent. elements must either be a tag:value dict or
           an ordered list of (tag, value) tuples."""
        for item in elements:
            if isinstance(item, tuple):
                tag, value = item
            else:
                tag = item
                value = elements[tag]
            if prefix:
                tag = prefix + ':' + tag
            e = parent.appendChild(self.doc.createElement(tag))
            e.appendChild(self.doc.createTextNode(value))
        return parent

    def set_attributes(self, element, attributes):
        """Set several attributes on element, from dict. attributes must either be an
           attr:value dict or an ordered list of (attr, value) tuples. If element is a
           string, then create an element with than name. Return (possibly created)
           element."""
        if isinstance(element, basestring):
            element = self.doc.createElement(element)
        for item in attributes:
            if isinstance(item, tuple):
                attr, value = item
            else:
                attr = item
                value = attributes[item]
            element.setAttribute(attr, value)
        return element

    def set_verb(self, verb):
        """Set the verb of the response. Should be called before any 'real' method such
           as identify(), get_record(), etc..."""
        self.verb = verb
        self.request.setAttribute('verb', self.verb)

    def identify(self):
        """Append Identify tag and child nodes"""

        identity = []
        identity[:] = self.identity[:]
        earliest = self.datasource.get_earliest_time()
        identity.insert(4, ('earliestDatestamp', iso_time(earliest)))

        group = self.root.appendChild(self.doc.createElement('Identify'))
        self.append_elements(group, identity)

    def error(self, code, msg = None):
        """Add error tag using code. If msg is not provided, use a default error message."""

        msgs = {
            'badArgument':              'Incorrect arguments',
            'badResumptionToken':       'Invalid resumption token',
            'badVerb':                  'Illegal OAI verb',
            'noSetHierarchy':           'This repository does not support sets.',
            'idDoesNotExist':           'No such record',
            'cannotDisseminateFormat':  'Unsupported metadata format',
            'noRecordsMatch':           'The request returned an empty record set'
        }

        if not msg:
            msg = msgs[code]
            if not msg:
                raise Exception("No such error code: %s" % code)

        err = self.root.appendChild(self.set_attributes('error', {'code': code}))
        err.appendChild(self.doc.createTextNode(msg))

    def make_record_header(self, id, ctime):
        """Build and return a record header"""
        header = self.doc.createElement('header')
        self.append_elements(header, [('identifier', id), ('datestamp', iso_time(ctime))])
        return header

    def make_record(self, id, dc, ctime):
        """Build and return a record"""
        record = self.doc.createElement('record')
        header = record.appendChild(self.make_record_header(id, ctime))
        metadata = record.appendChild(self.doc.createElement('metadata'))
        container = metadata.appendChild(self.doc.createElement('oai_dc:dc'))
        self.set_attributes(container, [
          ('xmlns:oai_dc',       "http://www.openarchives.org/OAI/2.0/oai_dc/"),
          ('xmlns:dc',           "http://purl.org/dc/elements/1.1/"),
          ('xmlns:xsi',          "http://www.w3.org/2001/XMLSchema-instance"),
          ('xsi:schemaLocation', "http://www.openarchives.org/OAI/2.0/oai_dc/ "
                                 "http://www.openarchives.org/OAI/2.0/oai_dc.xsd")
        ])
        self.append_elements(container, dc, prefix='dc')
        return record

    def parse_dc(self, data):
        valid = ['title', 'creator', 'subject', 'description', 'publisher', 'contributor',
                 'date', 'type', 'format', 'identifier', 'source', 'language', 'relation',
                 'coverage', 'rights']

        parsed = []
        id = None
        for k, v in data:
            try:
                valid.index(k)
                parsed.append((k, v))
                if k == 'identifier':
                    id = v
            except ValueError:
                pass

        return id, parsed


    def get_record(self, id):
        """Append GetRecord result"""
        try:
            record = self.datasource.get_record(id)
        except BadArgumentError, e:
            self.error('badArgument', e.message)
            return
        if not record:
            self.error('idDoesNotExist')
        else:
            dc, ctime = record
            dc_id, dc = self.parse_dc(dc)
            if dc_id == None:
                raise Exception("DataSource.get_record() didn't provide an 'identifier' dublin core element")
#            elif dc_id != id:
#                raise Exception("DataSource.get_record() returned an 'identifier' dublin core element "
#                                "which is different from the requested identifier")

            self.set_attributes(self.request, {'identifier': id, 'metadataPrefix': 'oai_dc'})
            container = self.root.appendChild(self.doc.createElement(self.verb))
            container.appendChild(self.make_record(id, dc, ctime))

    def format_id_header(self, id):
        organization = self.identity[0][1] 
        if 'http' in id:
            id = id.split('/')[-1]
        return ':'.join(['oai', organization.lower(), 'items', id])
    
    def list_records(self, from_time, until_time, token = None, ids_only = False):
        """Append ListIdentifiers or ListRecords result"""
        offset = 0
        if token:
            self.request.setAttribute('resumptionToken', token)
            try:
                from_time, until_time, offset = token.split(',')
            except ValueError:
                self.error('badResumptionToken')
                return

            if len(from_time):
                from_time = parse_iso_time(from_time)
                if not from_time:
                    self.error('badResumptionToken')
                    return
            else:
                from_time = None

            if len(until_time):
                until_time = parse_iso_time(until_time)
                if not until_time:
                    self.error('badResumptionToken')
                    return
            else:
                until_time = None

            try:
                offset = int(offset)
            except ValueError:
                self.error('badResumptionToken')
                return
        else:
            if from_time:
                self.request.setAttribute('from', iso_time(from_time))
            if until_time:
                self.request.setAttribute('until', iso_time(until_time))

        count = self.datasource.count_records(from_time, until_time)
        data = self.datasource.list_records(offset, self.max_records_per_response, from_time, until_time)
        if (len(data) > self.max_records_per_response):
            raise Exception("DataSource.list_records() returned too many records")

        if len(data):
            container = self.root.appendChild(self.doc.createElement(self.verb))
            for item in data:
                dc, ctime = item
                id, dc = self.parse_dc(dc)
                id = self.format_id_header(id)
                if id == None:
                    raise Exception("DataSource.list_records() didn't provide an 'identifier' dublin core element")

                if ids_only:
                    container.appendChild(self.make_record_header(id, ctime))
                else:
                    container.appendChild(self.make_record(id, dc, ctime))

            if count - offset > self.max_records_per_response:
                token = container.appendChild(self.doc.createElement('resumptionToken'))
                token.setAttribute('completeListSize', str(count))

                if from_time:
                    from_time = iso_time(from_time)
                else:
                    from_time = ''

                if until_time:
                    until_time = iso_time(until_time)
                else:
                    until_time = ''

                token_str = "%s,%s,%d" % (from_time, until_time, offset + len(data))
                token.appendChild(self.doc.createTextNode(token_str))
            elif offset:
                token = container.appendChild(self.doc.createElement('resumptionToken'))
                token.setAttribute('completeListSize', str(count))
        else:
            self.error("noRecordsMatch")

    def list_formats(self, id = None):
        """Append ListMetadataFormats result"""
        if id:
            record = self.datasource.get_record(id)
            if not record:
                self.error('idDoesNotExist')
                return
            self.request.setAttribute('identifier', id)

        container = self.root.appendChild(self.doc.createElement(self.verb))
        format = container.appendChild(self.doc.createElement('metadataFormat'))
        self.append_elements(format, [
            ('metadataPrefix',       'oai_dc'),
            ('schema',               'http://www.openarchives.org/OAI/2.0/oai_dc.xsd'),
            ('metadataNamespace',    'http://www.openarchives.org/OAI/2.0/oai_dc/')
        ])

    def free(self):
        """Free the resources used by this response"""
        try:
            self.doc.unlink()
        except AttributeError:
            # Apparently no free/unlink method in libxml2dom
            pass

class BadArgumentError(Exception):
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2011 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from django.contrib.auth.models import User
from telemeta.models.core import *
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _
import django.db.models as models
from django.forms import ModelForm


class Revision(ModelCore):
    "Revision made by user"

    ELEMENT_TYPE_CHOICES = (('collection', 'collection'), ('item', 'item'), ('part', 'part'), ('marker', 'marker'), ('media', 'media'), ('fonds', 'fonds'), ('corpus', 'corpus'))
    CHANGE_TYPE_CHOICES  = (('import', 'import'), ('create', 'create'), ('update', 'update'), ('delete','delete'))

    element_type         = CharField(_('element type'), choices=ELEMENT_TYPE_CHOICES, max_length=16, required=True)
    element_id           = IntegerField(_('element identifier'), required=True)
    change_type          = CharField(_('modification type'), choices=CHANGE_TYPE_CHOICES, max_length=16, required=True)
    time                 = DateTimeField(_('time'), auto_now_add=True)
    user                 = ForeignKey(User, db_column='username', related_name="revisions", verbose_name=_('user'))

    @classmethod
    def touch(cls, element, user):
        "Create or update a revision"
        revision = cls(element_type=element.element_type, element_id=element.pk, user=user, change_type='create')
        if element.pk:
            try:
                element.__class__.objects.get(pk=element.pk)
            except ObjectDoesNotExist:
                pass
            else:
                revision.change_type = 'update'

        revision.save()
        return revision

    def __str__(self):
        return str(self.time) + ' : ' + self.element_type + ' : ' + str(self.element_id)

    class Meta(MetaCore):
        db_table = 'revisions'
        ordering = ['-time']


class UserProfile(models.Model):
    "User profile extension"

    user            = ForeignKey(User, unique=True, required=True)
    institution     = CharField(_('Institution'))
    department      = CharField(_('Department'))
    attachment      = CharField(_('Attachment'))
    function        = CharField(_('Function'))
    address         = TextField(_('Address'))
    telephone       = CharField(_('Telephone'))
    expiration_date = DateField(_('Expiration_date'))

    class Meta(MetaCore):
        db_table = 'profiles'
        permissions = (("can_view_users_and_profiles", "Can view other users and any profile"),)


class Criteria(ModelCore):
    "Search criteria"

    element_type = 'search_criteria'

    key = CharField(_('key'), required=True)
    value = CharField(_('value'), required=True)

    class Meta(MetaCore):
        db_table = 'search_criteria'


class Search(ModelCore):
    "Keywork search"

    element_type = 'search'

    username = ForeignKey(User, related_name="searches", db_column="username")
    date = DateTimeField(_('date'), auto_now_add=True)
    description = CharField(_('Description'))
    criteria = models.ManyToManyField(Criteria, related_name="search", verbose_name=_('criteria'), blank=True, null=True)

    class Meta(MetaCore):
        db_table = 'searches'
        ordering = ['-date']

    def __unicode__(self):
        return ' - '.join([self.username.username, unicode(self.date), ' - '.join([c.key for c in self.criteria.all()])])



# -*- coding: utf-8 -*-
from telemeta.models import *
from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin

admin.site.unregister(User)

class MediaFondsAdmin(admin.ModelAdmin):
    search_fields = ['title', 'code']
    ordering = ['code']
    filter_horizontal = ['children']

class MediaCorpusAdmin(admin.ModelAdmin):
    search_fields = ['title', 'code']
    ordering = ['code']
    filter_horizontal = ['children']

class MediaCollectionRelatedInline(admin.StackedInline):
    model = MediaCollectionRelated

class MediaCollectionIdentifierInline(admin.StackedInline):
    model = MediaCollectionIdentifier
    max_num = 1

class MediaCollectionAdmin(admin.ModelAdmin):
    search_fields = ['title', 'code']
    ordering = ['code']
    inlines = [MediaCollectionIdentifierInline,
                MediaCollectionRelatedInline]

class MediaItemRelatedInline(admin.StackedInline):
    model = MediaItemRelated

class MediaItemMarkerInline(admin.StackedInline):
    model = MediaItemMarker

class MediaItemTranscodedInline(admin.StackedInline):
    model = MediaItemTranscoded

class MediaItemIdentifierInline(admin.StackedInline):
    model = MediaItemIdentifier
    max_num = 1

class MediaItemAdmin(admin.ModelAdmin):
    search_fields = ['title', 'code']
    ordering = ['code']
    exclude = ('copied_from_item', )
    inlines = [MediaItemIdentifierInline,
                MediaItemRelatedInline,
                MediaItemTranscodedInline,
                MediaItemMarkerInline]

class MediaPartAdmin(admin.ModelAdmin):
    search_fields = ['title', 'item__code']
    ordering = ['title']

class InstrumentAdmin(admin.ModelAdmin):
    search_fields = ['name']
    ordering = ['name']

class InstrumentAliasAdmin(admin.ModelAdmin):
    search_fields = ['name']
    ordering = ['name']

class InstrumentRelationAdmin(admin.ModelAdmin):
    search_fields = ['instrument__name', 'parent_instrument__name']
    ordering = ['parent_instrument__name']

class InstrumentAliasRelationAdmin(admin.ModelAdmin):
    search_fields = ['alias__name', 'instrument__name']
    ordering = ['alias__name']

class LocationAdmin(admin.ModelAdmin):
    search_fields = ['name']
    ordering = ['name']

class LocationAliasAdmin(admin.ModelAdmin):
    search_fields = ['location__name', 'alias']
    ordering = ['alias']

class LocationRelationAdmin(admin.ModelAdmin):
    search_fields = ['location__name', 'ancestor_location__name']
    ordering = ['ancestor_location__name']

class LanguageAdmin(admin.ModelAdmin):
    search_fields = ['name', 'identifier']
    ordering = ['name']

class RevisionAdmin(admin.ModelAdmin):
    search_fields = ['element_id', 'user']
    ordering = ['-time']

class FormatAdmin(admin.ModelAdmin):
    search_fields = ['original_code', 'tape_reference']

class UserProfileInline(admin.StackedInline):
	model = UserProfile

class UserProfileAdmin(UserAdmin):
	inlines = [UserProfileInline]

class PlaylistAdmin(admin.ModelAdmin):
    search_fields = ['title', 'public_id']

admin.site.register(MediaFonds, MediaFondsAdmin)
admin.site.register(MediaCorpus, MediaCorpusAdmin)
admin.site.register(MediaCollection, MediaCollectionAdmin)
admin.site.register(MediaItem, MediaItemAdmin)
admin.site.register(MediaPart, MediaPartAdmin)

admin.site.register(Instrument, InstrumentAdmin)
admin.site.register(InstrumentAlias, InstrumentAliasAdmin)
admin.site.register(InstrumentRelation, InstrumentRelationAdmin)
admin.site.register(InstrumentAliasRelation, InstrumentAliasRelationAdmin)

admin.site.register(Location, LocationAdmin)
admin.site.register(LocationType)
admin.site.register(LocationAlias, LocationAliasAdmin)
admin.site.register(LocationRelation, LocationRelationAdmin)

admin.site.register(Language, LanguageAdmin)

admin.site.register(Revision, RevisionAdmin)

admin.site.register(Format, FormatAdmin)

admin.site.register(User, UserProfileAdmin)

admin.site.register(PublisherCollection)
admin.site.register(Playlist, PlaylistAdmin)



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *


class PlaylistView(object):
    """Provide Playlist web UI methods"""

    @jsonrpc_method('telemeta.add_playlist')
    def add_playlist(request, playlist):
        # playlist must be a dict
        if isinstance(playlist, dict):
            m = Playlist()
            m.public_id = playlist['public_id']
            m.title = playlist['title']
            m.description = playlist['description']
            m.author = request.user
            m.save()
        else:
            raise 'Error : Bad playlist dictionnary'

    @jsonrpc_method('telemeta.del_playlist')
    def del_playlist(request, public_id):
        m = Playlist.objects.get(public_id=public_id)
        m.delete()

    @jsonrpc_method('telemeta.update_playlist')
    def update_playlist(request, playlist):
        if isinstance(playlist, dict):
            m = Playlist.objects.get(public_id=playlist['public_id'])
            m.title = playlist['title']
            m.description = playlist['description']
            m.save()
        else:
            raise 'Error : Bad playlist dictionnary'

    @jsonrpc_method('telemeta.add_playlist_resource')
    def add_playlist_resource(request, playlist_id, playlist_resource):
        # playlist_resource must be a dict
        if isinstance(playlist_resource, dict):
            m = PlaylistResource()
            m.public_id = playlist_resource['public_id']
            m.playlist = Playlist.objects.get(public_id=playlist_id, author=request.user)
            m.resource_type = playlist_resource['resource_type']
            m.resource_id = playlist_resource['resource_id']
            m.save()
        else:
            raise 'Error : Bad playlist_resource dictionnary'

    @jsonrpc_method('telemeta.del_playlist_resource')
    def del_playlist_resource(request, public_id):
        m = PlaylistResource.objects.get(public_id=public_id)
        m.delete()

    def get_elements(self, playlist, resource_type):
        resources = PlaylistResource.objects.filter(playlist=playlist)
        elements = []
        for resource in resources:
            if resource_type == 'items':
                if resource.resource_type == 'collection':
                    collections = MediaCollection.objects.filter(id=resource.resource_id)
                    if collections:
                        collection = collections[0]
                        collection_items = MediaItem.objects.filter(collection=collection)
                        for item in collection_items:
                            elements.append(item)
                elif resource.resource_type == 'item':
                    items = MediaItem.objects.filter(id=resource.resource_id)
                    if items:
                        item = items[0]
                        elements.append(item)
            elif resource_type == 'collections':
                if resource.resource_type == 'collection':
                    collection = MediaCollection.objects.get(id=resource.resource_id)
                    elements.append(collection)
        return elements

    def playlist_csv_export(self, request, public_id, resource_type):
        playlist = Playlist.objects.get(public_id=public_id)
        elements = self.get_elements(playlist, resource_type)
        pseudo_buffer = Echo()
        writer = UnicodeCSVWriter(pseudo_buffer, elements)
        response = StreamingHttpResponse(writer.output(), content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename='+playlist.title+'_'+resource_type+'.csv'
        return response



# -*- coding: utf-8 -*-
# Copyright (C) 2015 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *
from telemeta.models import *
from collections import OrderedDict
from ebooklib import epub
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _


class BaseEpubMixin(TelemetaBaseMixin):
    "Download corpus data embedded in an EPUB3 file"

    local_path = os.path.dirname(__file__)
    css = os.sep.join([local_path, '..', 'static', 'telemeta', 'css', 'telemeta_epub.css'])
    default_image = os.sep.join([local_path, '..', 'static', 'telemeta', 'images', 'cul_de_lampe.jpg'])
    default_image_end = os.sep.join([local_path, '..', 'static', 'telemeta', 'images', 'cul_de_lampe-fin_page.jpg'])
    template = os.sep.join([local_path, '..', 'templates', 'telemeta', 'inc', 'epub_collection.html'])
    template_preamble = os.sep.join([local_path, '..', 'templates', 'telemeta', 'inc', 'epub_preamble.html'])
    template_cover = os.sep.join([local_path, '..', 'templates', 'telemeta', 'inc', 'epub_cover.html'])

    def setup_epub(self, corpus, collection=None, path=None):
        self.book = epub.EpubBook()
        self.corpus = corpus
        self.collection = collection
        self.site = Site.objects.get_current()
        self.chapters = []
        self.default_image_added = False

        if not collection:
            self.filename = corpus.code
            self.book.set_title(corpus.title)
            self.full_title = corpus.title
        else:
            self.filename = collection.code
            short_title = collection.title.split(' ')
            if not ' 0' in collection.title:
                short_title = short_title[0][:4] + ' ' + short_title[1]
            else:
                short_title = 'Intro'
            self.book.set_title(corpus.title[:15] + '... ' + short_title)
            self.full_title = corpus.title + ' - ' + collection.title

        self.path = self.cache_data.dir + os.sep + self.filename + '.epub'
        return self.path

    def write_book(self):

        # add metadata
        self.book.set_identifier(self.corpus.public_id)
        self.book.set_language('fr')
        self.book.add_author(self.corpus.descriptions)

        # add css style
        style = open(self.css, 'r')
        css = epub.EpubItem(uid="style_nav", file_name="style/epub.css", media_type="text/css", content=style.read())
        self.book.add_item(css)

        if self.collection:
            self.collections = [self.collection]
            mode_single = True
            instance = self.collection
            if ' 0' in self.collection.title:
                chap_num = "d'introduction"
            else:
                chap_num = self.collection.code.split('_')[-1]
            context = {'title': 'chapitre ' + chap_num,
                        'mode_single': mode_single}
        else:
            self.collections = self.corpus.children.all()
            mode_single = False
            instance = self.corpus
            context = {'title': '', 'mode_single': mode_single}

        # add cover image
        for media in instance.related.all():
            self.cover_filename = os.path.split(media.file.path)[-1]
            self.book.set_cover(self.cover_filename, open(media.file.path, 'rb').read())
            break

        preamble = epub.EpubHtml(title='Copyright', file_name='copyright' + '.xhtml', lang='fr')
        preamble.content = render_to_string(self.template_preamble, context)
        preamble.is_chapter = True
        self.default_image_added = False
        default_image_relative_path = ''
        self.book.add_item(preamble)
        self.chapters.append(preamble)

        image = open(self.default_image_end, 'r')
        default_image_end_relative_path = 'images' + os.sep + os.path.split(self.default_image_end)[-1]
        epub_last_image = epub.EpubItem(file_name=default_image_end_relative_path,
                                        content=image.read())
        self.book.add_item(epub_last_image)
        i = 1

        for collection in self.collections:
            items = {}
            for item in collection.items.all():
                if '.' in item.old_code:
                    id = item.old_code.split('.')[1]
                else:
                    id = item.old_code
                for c in id:
                    if c.isalpha():
                        id = id.replace(c, '.' + str(ord(c)-96))
                items[item] = float(id)
            items = OrderedDict(sorted(items.items(), key=lambda t: t[1]))

            for item in items:
                if item.file:
                    audio = open(item.file.path, 'r')
                    filename = str(item.file)
                    epub_item = epub.EpubItem(file_name=str(item.file), content=audio.read())
                    self.book.add_item(epub_item)

                related_all = item.related.all()
                if related_all:
                    for related in related_all:
                        if 'image' in related.mime_type:
                            image = open(related.file.path, 'r')
                            epub_item = epub.EpubItem(file_name=str(related.file), content=image.read())
                            self.book.add_item(epub_item)
                elif not self.default_image_added:
                    image = open(self.default_image, 'r')
                    default_image_relative_path = 'images' + os.sep + os.path.split(self.default_image)[-1]
                    epub_item = epub.EpubItem(file_name=default_image_relative_path,
                                        content=image.read())
                    self.book.add_item(epub_item)
                    self.default_image_added = True

            title_split = collection.title.split(' - ')
            if len(title_split) > 1:
                if ' 0' in title_split[0]:
                    title = ''
                    subtitle = title_split[1]
                    chapter_title = subtitle
                else:
                    title = title_split[0]
                    subtitle = title_split[1]
                    chapter_title = ' - '.join([title, subtitle])
            else:
                title = collection.title
                subtitle = ''
                chapter_title = title

            last_collection = False
            if i == len(self.collections):
                last_collection = True

            context = {'collection': collection, 'title': title, 'subtitle': subtitle, 'mode_single': mode_single,
                        'site': self.site, 'items': items, 'default_image': default_image_relative_path,
                        'default_image_end': default_image_end_relative_path, 'last_collection': last_collection}
            c = epub.EpubHtml(title=chapter_title, file_name=collection.code + '.xhtml', lang='fr')
            c.content = render_to_string(self.template, context)
            self.book.add_item(c)
            self.chapters.append(c)
            i += 1

        # create table of contents
        # - add manual link
        # - add section
        # - add auto created links to chapters
        self.book.toc = (( self.chapters ))
        self.book.spine = self.chapters

        # add navigation files
        ncx = epub.EpubNcx()
        self.book.add_item(ncx)
        nav = epub.EpubNav()
        self.book.add_item(nav)
        if not mode_single:
            self.book.spine.insert(0,'nav')

        # create spin, add cover page as first page
        cover = epub.EpubHtml(title=self.full_title, file_name='cover-bis' + '.xhtml')
        cover.content = render_to_string(self.template_cover, {'image': self.cover_filename})
        self.book.add_item(cover)
        self.book.spine.insert(0, cover)

        # self.book.guide.insert(0, {
        # "type"  : "cover",
        # "href"  : cover.file_name,
        # "title" : cover.title,
        # })

        # write epub file
        epub.write_epub(self.path, self.book, {})






#!/usr/bin/python

import os, sys, string
import logging

class Logger:
    """A logging object"""

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)

log_file = 'thumbs.log'
logger = Logger(log_file)
root_dir = sys.argv[-1]
args = sys.argv[1:-1]
source_format = 'webm'
done = []
preview_tc = '00:00:05'

if os.path.exists(log_file):
    f = open(log_file, 'r')
    for line in f.readlines():
        done.append(line[:-1])
    f.close()

for root, dirs, files in os.walk(root_dir):
    for file in files:
        path = os.path.abspath(root + os.sep + file)
        name, ext = os.path.splitext(file)
        if ext[1:] == source_format:
            dest = os.path.abspath(root + os.sep + name + '.png')
            if not dest in done or '--force' in args:
                command = 'ffmpeg -ss '+ preview_tc + ' -i ' + path + '  -y ' + dest
                os.system(command)
                logger.logger.info(dest)

print "DONE!"



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012-2013 Guillaume Pellerin <yomguy@parisson.com>

# This file is part of TimeSide.

# TimeSide is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.

# TimeSide is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with TimeSide.  If not, see <http://www.gnu.org/licenses/>.

# Author: Guillaume Pellerin <yomguy@parisson.com>


from telemeta.util.xmltodict2 import *


class AutoFade(object):
    """ Automatically applies a fade in and a fade out trasitions between each segment of a KdenLive session.
        Each video clip needs to be splitted into one video track and an audio one ("Split audio"),
        so that an audio fade in/out is also applied.

        MLT files are also supported.
    """

    def __init__(self, path, audio_frames_out=2, audio_frames_in=1,
                       video_frames_out=3, video_frames_in=3):
        self.audio_frames_in = audio_frames_in
        self.audio_frames_out = audio_frames_out
        self.video_frames_in = video_frames_in
        self.video_frames_out = video_frames_out
        self.path = path
        self.session = xmltodict(self.path)

    def audio_fade_out(self, frame_out):
        child = {'attributes': {u'id': u'fadeout',
        u'in': unicode(int(frame_out)-self.audio_frames_out),
        u'out': unicode(frame_out)},
       'children': [{'attributes': {u'name': u'track'},
         'cdata': '0',
         'name': 'property'},
        {'attributes': {u'name': u'window'},
         'cdata': '75',
         'name': 'property'},
        {'attributes': {u'name': u'max_gain'},
         'cdata': '20dB',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_type'},
         'cdata': 'filter',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_service'},
         'cdata': 'volume',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_id'},
         'cdata': 'fadeout',
         'name': 'property'},
        {'attributes': {u'name': u'tag'},
         'cdata': 'volume',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_ix'},
         'cdata': '1',
         'name': 'property'},
        {'attributes': {u'name': u'gain'}, 'cdata': '1', 'name': 'property'},
        {'attributes': {u'name': u'end'}, 'cdata': '0', 'name': 'property'}],
       'name': 'filter'}

        return child

    def audio_fade_in(self, frame_in):
        child = {'attributes': {u'id': u'fadein',
        u'in': unicode(frame_in),
        u'out': unicode(int(frame_in)+self.audio_frames_in)},
       'children': [{'attributes': {u'name': u'track'},
         'cdata': '0',
         'name': 'property'},
        {'attributes': {u'name': u'window'},
         'cdata': '75',
         'name': 'property'},
        {'attributes': {u'name': u'max_gain'},
         'cdata': '20dB',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_type'},
         'cdata': 'filter',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_service'},
         'cdata': 'volume',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_id'},
         'cdata': 'fadein',
         'name': 'property'},
        {'attributes': {u'name': u'tag'},
         'cdata': 'volume',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_ix'},
         'cdata': '1',
         'name': 'property'},
        {'attributes': {u'name': u'gain'}, 'cdata': '0', 'name': 'property'},
        {'attributes': {u'name': u'end'}, 'cdata': '1', 'name': 'property'}],
       'name': 'filter'}

        return child


    def video_fade_out(self, frame_out):
        child = {'attributes': {u'id': u'fade_to_black',
        u'in': unicode(int(frame_out)-self.video_frames_out),
        u'out': unicode(frame_out)},
       'children': [{'attributes': {u'name': u'track'},
         'cdata': '0',
         'name': 'property'},
        {'attributes': {u'name': u'start'}, 'cdata': '1', 'name': 'property'},
        {'attributes': {u'name': u'mlt_type'},
         'cdata': 'filter',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_service'},
         'cdata': 'brightness',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_id'},
         'cdata': 'fade_to_black',
         'name': 'property'},
        {'attributes': {u'name': u'tag'},
         'cdata': 'brightness',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_ix'},
         'cdata': '1',
         'name': 'property'},
        {'attributes': {u'name': u'end'}, 'cdata': '0', 'name': 'property'}],
       'name': 'filter'}

        return child


    def video_fade_in(self, frame_in):
        child = {'attributes': {u'id': u'fade_from_black',
        u'in': unicode(frame_in),
        u'out': unicode(int(frame_in)+self.video_frames_in)},
       'children': [{'attributes': {u'name': u'track'},
         'cdata': '0',
         'name': 'property'},
        {'attributes': {u'name': u'start'}, 'cdata': '0', 'name': 'property'},
        {'attributes': {u'name': u'mlt_type'},
         'cdata': 'filter',
         'name': 'property'},
        {'attributes': {u'name': u'mlt_service'},
         'cdata': 'brightness',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_id'},
         'cdata': 'fade_from_black',
         'name': 'property'},
        {'attributes': {u'name': u'tag'},
         'cdata': 'brightness',
         'name': 'property'},
        {'attributes': {u'name': u'kdenlive_ix'},
         'cdata': '1',
         'name': 'property'},
        {'attributes': {u'name': u'end'}, 'cdata': '1', 'name': 'property'}],
       'name': 'filter'}

        return child

    def run(self):
        audio_count = 0
        video_count = 0
        
        for attr in self.session['children']:
            if 'playlist' in attr['name'] and 'children' in attr:
                for att in attr['children']:
                    if 'producer' in att['attributes'] and not 'children' in att:                        
                        producer = att['attributes']['producer']
                        if producer != 'black':
                        
                            frame_in = att['attributes']['in']
                            frame_out = att['attributes']['out']

                            if 'audio' in producer:
                                if not audio_count % 2:
                                    att['children'] = [self.audio_fade_out(frame_out)]
                                else:
                                    att['children'] = [self.audio_fade_in(frame_in)]
                                audio_count += 1


                            if 'video' in producer:
                                if not video_count % 2:
                                    att['children'] = [self.video_fade_out(frame_out)]
                                else:
                                    att['children'] = [self.video_fade_in(frame_in)]
                                video_count += 1

        return dicttoxml(self.session).encode('utf-8')





from django.contrib.auth.models import User
from django.template import Template, Context
from django.test import TestCase
from saved_searches.models import SavedSearch
from notes.models import Note


class SavedSearchTestCase(TestCase):
    def setUp(self):
        super(SavedSearchTestCase, self).setUp()
        self.user1 = User.objects.create_user('testy', 'test@example.com', 'test')
        # self.user1.is_active = True
        # self.user1.save()
        self.note1 = Note.objects.create(
            title='A test note',
            content='Because everyone loves test data.',
            author='Daniel'
        )
        self.note2 = Note.objects.create(
            title='Another test note',
            content='Something to test with.',
            author='John'
        )
        self.note3 = Note.objects.create(
            title='NEVER GOING TO GIVE YOU UP',
            content='NEVER GOING TO LET YOU DOWN.',
            author='Rick'
        )
    
    def test_usage(self):
        # Check the stat pages first.
        resp = self.client.get('/search_stats/most_recent/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual(len(resp.context['page'].object_list), 0)
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_popular/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual(len(resp.context['page'].object_list), 0)
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        
        # Sanity check.
        resp = self.client.get('/search/')
        self.assertEqual(resp.status_code, 200)
        
        # Run a couple searches.
        resp = self.client.get('/search/', data={'q': 'test'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 2)
        self.assertEqual(SavedSearch.objects.all().count(), 1)
        
        resp = self.client.get('/search/', data={'q': 'everyone'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 1)
        self.assertEqual(SavedSearch.objects.all().count(), 2)
        
        resp = self.client.get('/search/', data={'q': 'test data'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 1)
        self.assertEqual(SavedSearch.objects.all().count(), 3)
        
        resp = self.client.get('/search/', data={'q': 'test'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 2)
        self.assertEqual(SavedSearch.objects.all().count(), 4)
        
        # This shouldn't get logged.
        resp = self.client.get('/search/', data={'q': 'test', 'page': 2})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 2)
        self.assertEqual(len(resp.context['page'].object_list), 1)
        self.assertEqual(SavedSearch.objects.all().count(), 4)
        
        # Run a couple user searches.
        self.assertEqual(self.client.login(username='testy', password='test'), True)
        
        resp = self.client.get('/search/', data={'q': 'test'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 2)
        self.assertEqual(SavedSearch.objects.all().count(), 5)
        self.assertEqual(SavedSearch.objects.filter(user=self.user1).count(), 1)
        
        resp = self.client.get('/search/', data={'q': 'everyone'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 1)
        self.assertEqual(SavedSearch.objects.all().count(), 6)
        self.assertEqual(SavedSearch.objects.filter(user=self.user1).count(), 2)
        
        resp = self.client.get('/search/', data={'q': 'test'})
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['paginator'].count, 2)
        self.assertEqual(SavedSearch.objects.all().count(), 7)
        self.assertEqual(SavedSearch.objects.filter(user=self.user1).count(), 3)
        
        self.client.logout()
        
        # Verify the lists.
        self.assertEqual([ss['user_query'] for ss in SavedSearch.objects.most_recent()], [u'test', u'everyone', u'test data'])
        self.assertEqual([ss['times_seen'] for ss in SavedSearch.objects.most_recent()], [4, 2, 1])
        self.assertEqual([ss['times_seen'] for ss in SavedSearch.objects.most_recent(threshold=2)], [4, 2])
        self.assertEqual([ss['user_query'] for ss in SavedSearch.objects.most_recent(collapsed=False)], [u'test', u'everyone', u'test', u'test', u'test data', u'everyone', u'test'])
        self.assertEqual([ss['times_seen'] for ss in SavedSearch.objects.most_recent(collapsed=False)], [1, 1, 1, 1, 1, 1, 1])
        self.assertEqual([ss['user_query'] for ss in SavedSearch.objects.most_popular()], [u'test', u'everyone', u'test data'])
        self.assertEqual([ss['times_seen'] for ss in SavedSearch.objects.most_popular()], [4, 2, 1])
        self.assertEqual([ss['times_seen'] for ss in SavedSearch.objects.most_popular(threshold=2)], [4, 2])
        
        
        # Check to see if stats updated.
        resp = self.client.get('/search_stats/most_recent/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone', u'test data'])
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_recent/username/testy/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'].username, 'testy')
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone'])
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_recent/area/general/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], u'general')
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone', u'test data'])
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_popular/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone', u'test data'])
        self.assertEqual([ss['times_seen'] for ss in resp.context['page'].object_list], [4, 2, 1])
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_popular/username/testy/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'].username, 'testy')
        self.assertEqual(resp.context['by_search_key'], None)
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone'])
        self.assertEqual([ss['times_seen'] for ss in resp.context['page'].object_list], [2, 1])
        self.assertEqual(resp.context['paginator'].num_pages, 1)
        
        resp = self.client.get('/search_stats/most_popular/area/general/')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.context['by_user'], None)
        self.assertEqual(resp.context['by_search_key'], u'general')
        self.assertEqual([ss['user_query'] for ss in resp.context['page'].object_list], [u'test', u'everyone', u'test data'])
        self.assertEqual([ss['times_seen'] for ss in resp.context['page'].object_list], [4, 2, 1])
        self.assertEqual(resp.context['paginator'].num_pages, 1)


class TemplateTagTestCase(TestCase):
    def render(self, template_string, context={}):
        t = Template(template_string)
        c = Context(context)
        return t.render(c)


class MostRecentSearchesTestCase(TemplateTagTestCase):
    def setUp(self):
        super(MostRecentSearchesTestCase, self).setUp()
        self.saved1 = SavedSearch.objects.create(
            search_key='general',
            user_query='test'
        )
        self.saved2 = SavedSearch.objects.create(
            search_key='general',
            user_query='everyone'
        )
        self.saved3 = SavedSearch.objects.create(
            search_key='general',
            user_query='test data'
        )
        self.saved4 = SavedSearch.objects.create(
            search_key='events',
            user_query='test'
        )
        self.saved5 = SavedSearch.objects.create(
            search_key='general',
            user_query='test'
        )
        self.saved6 = SavedSearch.objects.create(
            search_key='general',
            user_query='everyone'
        )
    
    def test_correct_usage(self):
        temp = """{% load saved_searches_tags %}{% most_recent_searches as recent %}{% for search in recent %}'{{ search.user_query }}' {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'everyone' 'test' 'test data' ")
        
        temp = """{% load saved_searches_tags %}{% most_recent_searches as recent for_search_key "general" %}{% for search in recent %}'{{ search.user_query }}' {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'everyone' 'test' 'test data' ")
        
        temp = """{% load saved_searches_tags %}{% most_recent_searches as recent for_search_key "events" %}{% for search in recent %}'{{ search.user_query }}' {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'test' ")
        
        temp = """{% load saved_searches_tags %}{% most_recent_searches as recent limit 1 %}{% for search in recent %}'{{ search.user_query }}' {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'everyone' ")


class MostPopularSearchesTestCase(TemplateTagTestCase):
    def setUp(self):
        super(MostPopularSearchesTestCase, self).setUp()
        self.saved1 = SavedSearch.objects.create(
            search_key='general',
            user_query='test'
        )
        self.saved2 = SavedSearch.objects.create(
            search_key='general',
            user_query='everyone'
        )
        self.saved3 = SavedSearch.objects.create(
            search_key='general',
            user_query='test data'
        )
        self.saved4 = SavedSearch.objects.create(
            search_key='events',
            user_query='test'
        )
        self.saved5 = SavedSearch.objects.create(
            search_key='general',
            user_query='test'
        )
        self.saved6 = SavedSearch.objects.create(
            search_key='general',
            user_query='everyone'
        )
    
    def test_correct_usage(self):
        temp = """{% load saved_searches_tags %}{% most_popular_searches as popular %}{% for search in popular %}'{{ search.user_query }}' ({{ search.times_seen }}x) {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'test' (3x) 'everyone' (2x) 'test data' (1x) ")
        
        temp = """{% load saved_searches_tags %}{% most_popular_searches as popular for_search_key "general" %}{% for search in popular %}'{{ search.user_query }}' ({{ search.times_seen }}x) {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'everyone' (2x) 'test' (2x) 'test data' (1x) ")
        
        temp = """{% load saved_searches_tags %}{% most_popular_searches as popular for_search_key "events" %}{% for search in popular %}'{{ search.user_query }}' ({{ search.times_seen }}x) {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'test' (1x) ")
        
        temp = """{% load saved_searches_tags %}{% most_popular_searches as popular limit 1 %}{% for search in popular %}'{{ search.user_query }}' ({{ search.times_seen }}x) {% endfor %}"""
        context = {}
        output = self.render(temp, context)
        self.assertEqual(output, u"'test' (3x) ")



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2014 Parisson SARL

import random, re
from django.db.models import Q
from django.contrib.sites.models import Site


def get_random_hash():
    hash = random.getrandbits(64)
    return "%016x" % hash


def get_full_url(path):
    return 'http://' + Site.objects.get_current().domain + path


def word_search_q(field, pattern):
    words = re.split("[ .*-]+", pattern)
    q = Q()
    for w in words:
        if len(w) >= 3:
            kwargs = {field + '__icontains': w}
            q &= Q(**kwargs)

    return q



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
from telemeta.models import *
from telemeta.util.unaccent import unaccent
import logging
import codecs

class Command(BaseCommand):
    help = "Init original formats from a txt file (see files in example/init/"
    args = "path"
    admin_email = 'webmaster@parisson.com'

    def handle(self, *args, **options):
        path = args[0]
        file = open(path, 'r')
        for format in file.readlines():
            if not PhysicalFormat.objects.filter(value=format):
                format = PhysicalFormat(value=format)
                format.save()





